@@ -1,21 +1,28 @@
 import traceback
 import os.path
-from liquer.context import Context, find_queries_in_template
+from liquer.context import get_context, find_queries_in_template
 
 
 def evaluate(query):
     """Evaluate query, returns a State, cache the output in supplied cache"""
-    return Context().evaluate(query)
+    return get_context().evaluate(query)
 
 
-def evaluate_and_save(query, target_directory=None, target_file=None):
+def evaluate_and_save(
+    query, target_directory=None, target_file=None, target_resource_directory=None
+):
     """Evaluate query and save result.
     Output is saved either to
     - a target directory (current working directory by default) to a file deduced from the query, or
     - to target_file (if specified)
     Returns a state.
     """
-    return Context().evaluate_and_save(query, target_directory=target_directory, target_file=target_file)
+    return get_context().evaluate_and_save(
+        query,
+        target_directory=target_directory,
+        target_file=target_file,
+        target_resource_directory=target_resource_directory,
+    )
 
 
 def evaluate_template(template: str, prefix="$", sufix="$"):
@@ -23,4 +30,4 @@ def evaluate_template(template: str, prefix="$", sufix="$"):
     Queries in the template are delimited by prefix and sufix.
     Queries should evaluate to strings and should not cause errors.
     """
-    return Context().evaluate_template(template, prefix=prefix, sufix=sufix)
\ No newline at end of file
+    return get_context().evaluate_template(template, prefix=prefix, sufix=sufix)
